package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.FirePointChannelConfigDao;
import com.xkrs.dao.FirePointOrdinaryDao;
import com.xkrs.dao.StreetDao;
import com.xkrs.dao.SysUserDao;
import com.xkrs.helper.FirePointPushManager;
import com.xkrs.helper.FirePointQueryManager;
import com.xkrs.helper.GeoCodeHelper;
import com.xkrs.model.bean.DataWrapper2;
import com.xkrs.model.entity.FirePointOrdinaryEntity;
import com.xkrs.model.entity.StreetEntity;
import com.xkrs.model.qo.AllFirePointQo;
import com.xkrs.model.qo.UpdateFirePointQo;
import com.xkrs.service.DispatchFirePointService;
import com.xkrs.utilsnew.DateTimeUtils;
import com.xkrs.utilsnew.FirePointFilterUtils;
import org.apache.hc.core5.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@Service
public class DispatchFirePointServiceImpl implements DispatchFirePointService {

    public static Logger log = LoggerFactory.getLogger(DispatchFirePointServiceImpl.class);

    private final Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private FirePointChannelConfigDao channelConfigDao;

    @Resource
    private FirePointOrdinaryDao firePointOrdinaryDao;

    @Resource
    private FirePointQueryManager firePointQueryManager;

    @Resource
    private FirePointPushManager firePointPushManager;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private StreetDao streetDao;

    /**
     * 推送火点到秸秆系统
     */
    @Override
    public String insertFirePoint(AllFirePointQo firePointQo) {
        DataWrapper2<Boolean, String> checkLandTypeWrapper = FirePointFilterUtils.checkLandType(firePointQo.getLandtype());
        if (!checkLandTypeWrapper.getData1()) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, checkLandTypeWrapper.getData2(), locale);
        }
        DataWrapper2<Boolean, String> checkSatelliteTypeWrapper = FirePointFilterUtils.checkSatelliteType(firePointQo.getSatelliteType(), firePointQo.getConfidence());
        if (!checkSatelliteTypeWrapper.getData1()) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, checkSatelliteTypeWrapper.getData2(), locale);
        }

        log.info("insertFirePoint 接收到火点信息：" + firePointQo.toString());

        String fireCode = firePointQo.getFireCode();
        Long satelliteTimeTs = firePointQo.getSatelliteTimeTs();
        Double longitude = firePointQo.getLongitude();
        Double latitude = firePointQo.getLatitude();
        String satelliteType = firePointQo.getSatelliteType();
        String landType = firePointQo.getLandtype();
        String confidence = firePointQo.getConfidence();
        String fireImage = firePointQo.getFireImage();
        String satelliteImage = firePointQo.getSatelliteImage();
        String countyCode = String.valueOf(firePointQo.getCountyCode());
        String countyName = firePointQo.getCountyName();
        String townCode = firePointQo.getTowncode();

        LocalDateTime satelliteLocalDateTime = LocalDateTime.ofEpochSecond(satelliteTimeTs, 0, ZoneOffset.ofHours(8));
        String satelliteTime = DateTimeUtils.localDateTimeToString(satelliteLocalDateTime);//卫星时间
        String addTime = DateTimeUtils.localDateTimeToString(LocalDateTime.now());//添加时间

        FirePointOrdinaryEntity firePointOrdinaryEntity = new FirePointOrdinaryEntity();
        firePointOrdinaryEntity.setFireCode(fireCode);
        firePointOrdinaryEntity.setSatelliteTime(satelliteTime);
        firePointOrdinaryEntity.setLongitude(longitude);
        firePointOrdinaryEntity.setLatitude(latitude);
        firePointOrdinaryEntity.setSatelliteType(satelliteType);
        firePointOrdinaryEntity.setLandType(landType);
        firePointOrdinaryEntity.setConfidence(confidence);
        firePointOrdinaryEntity.setFireImage(fireImage);
        firePointOrdinaryEntity.setSatelliteImage(satelliteImage);
        firePointOrdinaryEntity.setAddTime(addTime);
        if (FirePointFilterUtils.checkDuplicated(firePointOrdinaryDao, firePointOrdinaryEntity)) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_EXIT, "该火点和历史火点重复", locale);
        }
        StreetEntity streetEntity = obtainInfoByCountyCode(countyCode);
        if (streetEntity != null) {
            firePointOrdinaryEntity.setProCode(streetEntity.getProCode());
            firePointOrdinaryEntity.setProName(streetEntity.getProName());
            firePointOrdinaryEntity.setCityCode(streetEntity.getCityCode());
            firePointOrdinaryEntity.setCityName(streetEntity.getCityName());
        }
        firePointOrdinaryEntity.setCountyCode(countyCode);
        firePointOrdinaryEntity.setCountyName(countyName);
        firePointOrdinaryEntity.setTownCode(townCode);
        bindFirePointAddress(firePointOrdinaryEntity);
        firePointOrdinaryDao.save(firePointOrdinaryEntity);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功", locale);
    }

    private StreetEntity obtainInfoByCountyCode(String countyCode) {
        try {
            return streetDao.findByCountyCode(countyCode).get(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 绑定火点位置
     * 如果能接收到TownCode（山东省内），会判断CountyCode、CountyName、TownCode，
     * 如果不能接收到TownCode（其他省份），会判断CountyCode、CountyName，
     * <p>
     * 如果根据经纬度反算出来的变量都一致，就更新TownCode、TownName、FirePointAddress，无备注。
     * 如果不一致，就只更新FirePointAddress，备注存储差异信息
     */
    private void bindFirePointAddress(FirePointOrdinaryEntity firePointOrdinaryEntity) {
        String countyCode = firePointOrdinaryEntity.getCountyCode();
        String countyName = firePointOrdinaryEntity.getCountyName();
        String townCode = firePointOrdinaryEntity.getTownCode();
        GeoCodeHelper geoCodeHelper = new GeoCodeHelper();
        geoCodeHelper.doGeoCode(firePointOrdinaryEntity.getLongitude(), firePointOrdinaryEntity.getLatitude());
        boolean geoCodeResult = geoCodeHelper.checkGeoCodeResult(countyCode, countyName, townCode);
        if (geoCodeResult) {
            firePointOrdinaryEntity.setTownCode(geoCodeHelper.getCalTownCode());
            firePointOrdinaryEntity.setTownName(geoCodeHelper.getCalTownName());
            firePointOrdinaryEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            firePointOrdinaryEntity.setRemark("");
        } else {
            firePointOrdinaryEntity.setTownCode(geoCodeHelper.getCalTownCode());
            firePointOrdinaryEntity.setTownName(geoCodeHelper.getCalTownName());
            firePointOrdinaryEntity.setFirePointAddress(geoCodeHelper.getCalFirePointAddress());
            String remark = "接收到的countyCode=" + countyCode + "，计算得到的countyCode=" + geoCodeHelper.getCalCountyCode() + "。接收到的countyName=" + countyName + "，计算得到的countyName=" + geoCodeHelper.getCalCountyName() + "。接收到的townCode=" + townCode + "，计算得到的townCode=" + geoCodeHelper.getCalTownCode() + "。";
            firePointOrdinaryEntity.setRemark(remark);
        }
    }

    /**
     * 更新火点状态
     */
    @Override
    public String updateFirePoint(UpdateFirePointQo updateFirePointQo) {
        List<UpdateFirePointQo.DataDTO> list = updateFirePointQo.getData();
        for (UpdateFirePointQo.DataDTO item : list) {
            String fireCode = item.getFireCode();
            String status = item.getStatus();
            if ((!TextUtils.isEmpty(fireCode)) && (!TextUtils.isEmpty(status)) && (("0".equals(status)) || ("1".equals(status)))) {
                System.out.println("参数正常：fireCode = " + fireCode + "，status = " + fireCode);
            } else {
                System.out.println("参数异常：fireCode = " + fireCode + "，status = " + fireCode);
                return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL, "更新失败", locale);
            }
        }
        for (UpdateFirePointQo.DataDTO item : list) {
            try {
                String fireCode = item.getFireCode();
                String status = item.getStatus();
                Specification<FirePointOrdinaryEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
                    //查询条件集合
                    List<Predicate> predicateList = new ArrayList<>();
                    predicateList.add(criteriaBuilder.equal(root.get("fireCode").as(String.class), fireCode));
                    Predicate[] predicateArray = new Predicate[predicateList.size()];
                    return criteriaBuilder.and(predicateList.toArray(predicateArray));
                };
                List<FirePointOrdinaryEntity> firePointList = firePointOrdinaryDao.findAll(specification, Sort.by(Sort.Direction.DESC, "satelliteTime"));
                if (firePointList.isEmpty()) {
                    System.out.println("操作异常，审核系统发送的fireCode在秸秆系统已有的火点中找不到：fireCode = " + fireCode + "，status = " + fireCode);
                } else {
                    for (FirePointOrdinaryEntity firePoint : firePointList) {
                        firePoint.setFireStatus(status);
                        firePointOrdinaryDao.save(firePoint);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "更新失败", locale);
            }
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "更新成功", locale);
    }

}
