package com.xkrs.utilsnew;

import com.xkrs.dao.FirePointOrdinaryDao;
import com.xkrs.model.bean.DataWrapper2;
import com.xkrs.model.entity.FirePointOrdinaryEntity;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

public class FirePointFilterUtils {

    private FirePointFilterUtils() {
    }

    public static DataWrapper2<Boolean, String> checkLandType(String landType) {
        if ("耕地".equals(landType)) {
            return new DataWrapper2<>(Boolean.TRUE, "");
        }
        return new DataWrapper2<>(Boolean.FALSE, "添加失败，只接收耕地类型的火点");
    }

    public static DataWrapper2<Boolean, String> checkSatelliteType(String satelliteType, String confidence) {
        if (satelliteType.startsWith("Himawari")) {
            if ("L".equals(confidence)) {
                return new DataWrapper2<>(Boolean.FALSE, "添加失败，不接收葵花低置信度火点");
            }
        }
        return new DataWrapper2<>(Boolean.TRUE, "");
    }

    public static boolean checkDuplicated(FirePointOrdinaryDao firePointDao, FirePointOrdinaryEntity firePointEntity) {
        Specification<FirePointOrdinaryEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("fireCode").as(String.class), firePointEntity.getFireCode()));
            predicateList.add(criteriaBuilder.equal(root.get("longitude").as(Double.class), firePointEntity.getLongitude()));
            predicateList.add(criteriaBuilder.equal(root.get("latitude").as(Double.class), firePointEntity.getLatitude()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteTime").as(String.class), firePointEntity.getSatelliteTime()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), firePointEntity.getSatelliteType()));
            predicateList.add(criteriaBuilder.equal(root.get("landType").as(String.class), firePointEntity.getLandType()));
            return criteriaBuilder.and(predicateList.toArray(new Predicate[predicateList.size()]));
        };
        List<FirePointOrdinaryEntity> list = firePointDao.findAll(specification);
        return list.size() > 0;
    }

}
